#
# Main file of HmotSoft
#

cat("Wellcome!\n")

cat("Loading libraries...")
library(DBI)
library(RMySQL)
library(ggplot2)
library(grid)
cat("OK\n")

cat("Loading configuration...")
source("./conf.R")
cat("OK\n")

cat("Closing old connections...")
conns <- dbListConnections(MySQL())
for(c in conns)
{
  dbDisconnect(c)
}
cat("OK\n")

if (mysqlHasDefault())
{
  
  cat("Connecting to database...")
  conn <- dbConnect(RMySQL::MySQL(), host = mysql_host, user = mysql_user, password = mysql_pass, dbname = mysql_db) 
  cat("OK\n")  
  
  cat("Creating test table...")
  dbSendQuery( conn, "drop table if exists `test`" )
  dbSendQuery( conn, paste("create table `test` (unique(`id`)) select `id`, `datetime`, `length`, `total_weight` from ", mysql_table, " where date(`datetime`) > '2014-07-28' and ", where ))
  cat("OK\n")
  
  cat("Fetching data info...")
  info <- dbGetQuery( conn, "select count(*) count, date_format(min(`datetime`),'%e. %c. %Y') `from`, date_format(max(`datetime`),'%e. %c. %Y') `to`, datediff(max(`datetime`), min(`datetime`)) `days` from test")
  total = info[1,1]
  cat("OK\n")
  
  cat("Data info:\n")
  print( info )
  
  cat("Loading data...")
  dataBelow <- dbGetQuery( conn, paste("select length/10 from test where total_weight < ", restriction))[,1]
  dataAbove <- dbGetQuery( conn, paste("select length/10 from test where total_weight >= ", restriction))[,1]
  cat("OK\n")  
  
  totalA <- length(dataAbove)
  totalB <- length(dataBelow)
  
  ratioB <- totalB/total;
  ratioA <- totalA/total;
  
  cat("Data below: ", totalB, "\n")
  cat("Data above: ", totalA, "\n")
  
  cat("Showing histogram...")
  p1 <- hist(dataAbove, border=FALSE)
  p2 <- hist(dataBelow, border=FALSE)
  xlim = range(p1$breaks,p2$breaks)
  ylim = range(0,p1$density,p2$density)
  plot( p1, col=rgb(1,0,0,1/4), main = "Lengh Histogram", xlab = "l[m]", freq = F, xlim=xlim,ylim=ylim)
  plot( p2, col=rgb(0,1,0,1/4), add=T, main = "Lengh Histogram", xlab = "l[m]" , freq = F, xlim=xlim,ylim=ylim)
  legend("topright", c("below","above"), fill = c("red","green"))
  cat("OK\n")
  
  cat("Evaluating data parameters...")  
  meanB = mean(dataBelow);
  meanA = mean(dataAbove);
  sdB = sd(dataBelow);
  sdA = sd(dataAbove);  
  cat("OK\n")
  
  cat("Showing gaussian curve...")  
  x <- seq(xlim[1],xlim[2],length=10000)
  y1 <- dnorm(x,mean=meanA, sd=sdA) * ratioA
  y2 <- dnorm(x,mean=meanB, sd=sdB) * ratioB
  plot(x,y1, type="l", lwd=1, col="red", xlab = "l[m]",ylab="p",xlim=xlim,ylim=ylim,main="Length vs. Probability")
  lines(x,y2, type="l", lwd=1, col="green")
  legend("topright", c("below","above"), fill = c("red","green"))  
  cat("OK\n")
  
  cat("Finding optimal length...")  
  for( i in 1:length(x) )
  {
    if( x[i] > meanB & x[i] < meanA & abs( y1[i] - y2[i] ) < 0.0001 )
    {
      length = x[i]
      break;      
    }
  }
  cat("OK\n")
  
  cat( "Length = ", length, "\n")
  
  cat("Evaluating error...")
  fn <- dbGetQuery( conn, paste("select count(*) from `test` where total_weight < ", restriction, " and length > ", length*10 ) )[,1]/total
  fp <- dbGetQuery( conn, paste("select count(*) from `test` where total_weight >= ", restriction, " and length <= ", length*10 ) )[,1]/total
  tn <- ratioA - fp
  tp <- ratioB - fn
  c <- (tp + tn)
  i <- (fp + fn);
  cat("OK\n")
  
  cat("True Positive", tp*100, "%\n")
  cat("False Negative", fn*100, "%\n")
  cat("True Negative", tn*100, "%\n")
  cat("False Positive", fp*100, "%\n")
  cat("Correct", c*100, "%\n")
  cat("Incorrect", i*100, "%\n")
  
  cat("Cleaning database...")
  dbSendQuery( conn, "drop table if exists `test`" )
  cat("OK\n")
  
  cat("Closing connection to database...")
  dbDisconnect(conn)
  cat("OK\n")
}




