#!/usr/bin/python

import MySQLdb as mdb
import numpy as np
from sklearn import svm
from sklearn import preprocessing
import os
import sys

class Main:

    table = "_3402_1017"
    limit1 = 5
    limit2 = 5
    columns = "length,axles_count"
    restriction = 12000
    pRatio = 0.67
    nRatio = 0.33
    C = 0.95
    kernel = "rbf"
    notify = False
    output = "stdout"
    groupIndex = 0
    groupNum = 3

    def readArguments(self):
        argc = len(sys.argv)

        for i in range( 1, argc ):
            arg = sys.argv[i]
            next = sys.argv[i+1] if i + 1 < argc else None

            if arg == "--table":
                self.table = next
            elif arg == "--limit1":
                self.limit1 = int(next)
            elif arg == "--limit2":
                self.limit2 = int(next)
            elif arg == "--columns":
                self.columns = next
            elif arg == "--restriction":
                self.restriction = int(next)
            elif arg == "--pRatio":
                self.pRatio = float(next)
            elif arg == "--nRatio":
                self.nRatio = float(next)
            elif arg == "--kernel":
                self.kernel = next
            elif arg == "--C":
                self.C = float(next)
            elif arg == "--output":
                self.output = next
            elif arg == "--notify":
                self.notify = next.lower() in ("yes", "true", "t", "1")
            elif arg == "--groupNum":
                self.groupNum = next
            elif arg == "--groupIndex":
                self.groupIndex = next

    def getDatabaseConnection(self):
        return mdb.connect(host="localhost", user="root", passwd="root", db="identifikator")


    def getTrainingData(self):
        db = self.getDatabaseConnection()
        cur = db.cursor()

        cur.execute( "select " + self.columns + " from " + self.table + " where date(`datetime`) > '2014-07-28' and total_weight < " + `self.restriction` + " and id % " + `self.groupNum` + " = " + `self.groupIndex` + " limit 0, " + `self.limit1`)
        dataBellow = cur.fetchall()

        cur.execute( "select " + self.columns + " from " + self.table + " where date(`datetime`) > '2014-07-28' and total_weight >= " + `self.restriction` + " and id % " + `self.groupNum` + " = " + `self.groupIndex` + " limit 0, " + `self.limit1`)
        dataAbove = cur.fetchall()

        db.close()

        x = dataBellow + dataAbove

        y = [1] * len(dataBellow) + [0] * len(dataAbove)

        return (x, y)


    def getTestingData(self):
        db = self.getDatabaseConnection()
        cur = db.cursor()

        cur.execute( "select " + self.columns + " from " + self.table + " where date(`datetime`) > '2014-07-28' and total_weight < " + `self.restriction` + " and id % " + `self.groupNum` + " = " + `self.groupIndex` + " limit " + `self.limit1` + "," + `self.limit2`)
        dataBellow = cur.fetchall()

        cur.execute( "select " + self.columns + " from " + self.table + " where date(`datetime`) > '2014-07-28' and total_weight >= " + `self.restriction` + " and id % " + `self.groupNum` + " = " + `self.groupIndex` + " limit " + `self.limit1` + "," + `self.limit2`)
        dataAbove = cur.fetchall()

        db.close()

        return (dataBellow, dataAbove)


    def printResults(self, negative, positive, true):
        if true:
            tn = negative * self.nRatio
            tp = positive * self.pRatio
            fn = self.pRatio - tp
            fp = self.nRatio - tn

        else:
            fn = negative * self.pRatio
            fp = positive * self.nRatio
            tn = self.nRatio - fp
            tp = self.pRatio - fn

        c = tp + tn
        i = fp + fn

        print( "Results:" )
        print "    True Positive:  " + self.toPercent(tp)
        print "    False Negative: " + self.toPercent(fn)
        print "    True Negative:  " + self.toPercent(tn)
        print "    False Positive: " + self.toPercent(fp)
        print "    Correct:        " + self.toPercent(c)
        print "    Incorrect:      " + self.toPercent(i)


    def toPercent(self, decimal):
        return `round(decimal * 100, 2)` + "%"

    def run(self):
        # Initializing...

        self.readArguments()

        #Fetching training data..."

        testData = self.getTrainingData()

        x = testData[0]
        y = testData[1]

        # Preprocessing training data..."

        scaler = preprocessing.StandardScaler().fit(x)
        x = scaler.transform(x)

        # Initializing SVM...

        clf = svm.SVC()
        #clf.cache_size = 300

        clf.C = self.C

        clf.class_weight = {1: self.pRatio, 0: self.nRatio}
        #clf.max_iter = 100000
        clf.verbose = False

        clf.kernel = self.kernel

        # Training SVM...

        clf.fit(x, y)

        # Fetching testing data...

        trainData = self.getTestingData()
        x1 = trainData[0]
        x2 = trainData[1]

        # Preprocessing testing data...
        x1 = scaler.transform(x1)
        x2 = scaler.transform(x2)

        # Testing SVM...

        y1 = clf.predict(x1)
        y2 = clf.predict(x2)

        tn = np.sum(y2 == 0) / float(len(y2))
        tp = np.sum(y1 == 1) / float(len(y1))

        if self.output == "stdout":
            self.printResults(tn,tp,True)
        elif self.output == "Mathematica":
            print( "{" + `tn` + "," + `tp` + "}" )

        if( self.notify ):
            os.system("kdialog --title 'SVM clasification' --icon 'task-complete' --passivepopup 'Finished.' 5")


if __name__ == '__main__':

        main = Main()
        main.run()

